@@ -1,16 +1,92 @@
+#' Obtain Critical Value Table 
+#'
+#' Obtain critical values of the Khmaladze martingale transformation test for various significane levels and sample sizes
+#'@param strDist  the name of the null distribution for the hypothesis test: Normal, Cauchy, or Logistic. Other distributions such as Gumbel, Weibull and Frechet will be available in later versions. 
+#'@param Modified a logical value which specifies whether or not to use the modeifed version of the test: False calls the original version while True calls the modified version.
+#'@return A 10-by-6 table of critical values for various significance levles (0.1, 0.075, 0.05, 0.025, 0.01) and sample sizes (10,20,...,100). 
+#'
+#'@examples
+#'
+#'
+#'
+#'### Critical values of the original test for a normal distribution
+#'
+#'
+#'strDist = "Normal"
+#'Modified=FALSE
+#'CritValue = GetCV(strDist, Modified)
+#'
+#'
+#'
+#'## Critical values of the modified test for the logistic distribution
+#'strDist = "Logistic"
+#'Modified=TRUE
+#'CritValue = GetCV(strDist, Modified)
+#'
+#'
+#'
+#'##Critical values of the modified test for the Cauchy distribution
+#'strDist = "Cauchy"
+#'Modified=TRUE
+#'CritValue = GetCV(strDist, Modified)
+#'
+#'@export
+#'@seealso KhmaladzeTrans()
+#'@useDynLib GofKmt
+
+
+GetCV = function(strDist, Modified){
+  
+  
+  if(Modified==FALSE){
+    strName = "Original"
+  }else{
+    strName = "Modified"
+  }
+  
+  if(strDist=="Normal"){
+    strDist="Normal"
+  }else if(strDist=="Logistic"){
+    strDist="Logistic"
+  }else if(strDist=="Cauchy"){
+    strDist="Cauchy"
+  }else{
+    strDist="Normal"
+  }
+  
+  targetstr = paste("Critical.Value.for.", strName, "Test.", strDist, sep="")
+  
+  print(targetstr)
+  CritMat = Tables[[targetstr]]
+  return(CritMat)
+}
+
+
+
 
 #' Implementing Khmaladze Martingale Transformation.
 #'
 #' Performs goodness-of-fit test through Khmaladze matringale transformation
 #'@param X  a random sample of n observations
-#'@param strDist  the name of the null distribution for the hypothesis test: Normal, Cauchy, or Logistic. Other distributions such as gamma, Weibull and Frechet will be available in later versions. 
+#'@param Modified a logical value which specifies whether or not to use the modeifed version of the test: False calls the original version while True calls the modified version.  
+#'@param strDist  the name of the null distribution for the hypothesis test: Normal, Cauchy, or Logistic. Other distributions such as Gumbel, Weibull and Frechet will be available in later versions. 
 #'@param bGraph  a logical value which specifies whether or not to get the graph of the objective function of the martingale transformation.
 #'@param nNum  the number of ticks on each segmented interval when drawing the graph of the objective function. The default is 10. Bigger value will result in a smoother graph.
 #'@return A list of the following values: 
 #'\describe{
-#'\item{opt.x}{ the value of x where the optimum of the objective function - which is also the test statistic - occurs.} 
+#'\item{opt.x}{ 
+#'\itemize{
+#'  \item When Modified is False, opt.x is the value of x where the optimum of the objective function - which is also the test statistic - occurs.
+#'  \item When Modified is True, opt.x is the vector of the value of x's where the infimum and supremum of \eqn{U_{n}} occur. 
+#'}
+#'} 
 #'
-#'\item{test.stat}{ the test statistic obtained through Khmaladze martingale transformation}
+#'\item{test.stat}{ 
+#'\itemize{
+#'  \item When Modified is False, test.stat is the test statistic obtained through Khmaladze martingale transformation.
+#'  \item When Modified is True, test.stat is the vector of the supremum of \eqn{U_{n}}, the infimum of \eqn{U_{n}}, and the difference of them. 
+#'}
+#'}
 #'
 #'\item{graph.data}{ a data frame which includes the information of the objective function.}
 #'
@@ -18,8 +94,6 @@
 #'
 #'\item{intervals}{ a list of segmented intervals over which the graph of the objective function is defined.}
 #'
-#'\item{crit.value}{ a vector of critical values for the level of 0.01, 0.025, 0.05, and 0.10}
-#'
 #'\item{mu}{ the point estimate for the location parameter mu}
 #'
 #'\item{sigma}{ the point estimate for the scale parameter sigma}
@@ -30,42 +104,65 @@
 #'n = 10
 #'X = rnorm(n, 1,3)    # Generate a random sample of n observations from N(1,3)
 #'strDist = "Normal"
-#'lResult = KhmaladzeTrans(X, strDist, bGraph=TRUE, nNum=10)
+#'Modified=FALSE
+#'lResult = KhmaladzeTrans(X, Modified, strDist, bGraph=TRUE, nNum=10)
 #'KMT_OptimalX = lResult$opt.x
 #'KMT_TestStat = lResult$test.stat
 #'KMT_DM = lResult$graph.data
 #'KMT_Graph = lResult$graph
+#'
+#'#### Draw the graph of the objective function
+#'KMT_Graph
+#'
 #'KMT_Intervals = lResult$intervals
-#'KMT_CriticalValue = lResult$crit.value
 #'KMT_Muhat = lResult$mu
 #'KMT_Sigmahat = lResult$sigma
+#'
+#'
 #'#####################
 #'
 #'#####################
 #'n = 10
 #'X = rlogis(n, 1,2)  # Generate a random sample of n observations from the logistic distribution
 #'strDist = "Logistic"
-#'lResult = KhmaladzeTrans(X, strDist, bGraph=TRUE, nNum=10)
-#'KMT_OptimalX = lResult$opt.x
-#'KMT_TestStat = lResult$test.stat
+#'Modified=TRUE
+#'lResult = KhmaladzeTrans(X, Modified, strDist, bGraph=TRUE, nNum=10)
+#'KMT_Optimal_Positive_X = lResult$opt.x[1]
+#'KMT_Optimal_Negative_X = lResult$opt.x[2]
+#'KMT_Postive_TestStat = lResult$test.stat[1]
+#'KMT_Negative_TestStat = lResult$test.stat[2]
+#'KMT_TestStat = lResult$test.stat[3]
 #'KMT_DM = lResult$graph.data
 #'KMT_Graph = lResult$graph
+#'
+#'#### Draw the graph of the objective function
+#'KMT_Graph
+#'
 #'KMT_Intervals = lResult$intervals
-#'KMT_CriticalValue = lResult$crit.value
 #'KMT_Muhat = lResult$mu
 #'KMT_Sigmahat = lResult$sigma
 #'#####################
+#'
+#'
+#'
+#'
+#'
 #'#####################
 #'n = 10
 #'X = rcauchy(n, 0,1)  # Generate a random sample of n observations from Cauchy distribution
 #'strDist = "Cauchy"
-#'lResult = KhmaladzeTrans(X, strDist, bGraph=TRUE, nNum=10)
+#'Modified=FALSE
+#'lResult = KhmaladzeTrans(X, Modified, strDist, bGraph=TRUE, nNum=10)
 #'KMT_OptimalX = lResult$opt.x
 #'KMT_TestStat = lResult$test.stat
 #'KMT_DM = lResult$graph.data
 #'KMT_Graph = lResult$graph
+#'
+#'#### Draw the graph of the objective function
+#'KMT_Graph
+#'
+#'
 #'KMT_Intervals = lResult$intervals
-#'KMT_CriticalValue = lResult$crit.value
 #'KMT_Muhat = lResult$mu
 #'KMT_Sigmahat = lResult$sigma
 #'#####################
@@ -79,12 +176,13 @@
 #'@references
 #'[2] E.V. Khmaladze, H.L. Koul (2009). Goodness-of-fit problem for errors in nonparametric regression: distribution free approach. Ann. Statist., 37(6A) 3165-3185.
 #'@references
-#'[3] Kim, Jiwoong (2020). Implementation of a goodness-of-fit test through Khmaladze martingale transformation.
+#'[3] Kim, Jiwoong (2020). Implementation of a goodness-of-fit test through Khmaladze martingale transformation. Comp. Stat., 35(4): 1993-2017    
 #'@export
+#'@importFrom Rcpp evalCpp
 #'@useDynLib GofKmt
 
 
-KhmaladzeTrans = function(X, strDist, bGraph=FALSE, nNum=10){
+KhmaladzeTrans = function(X, Modified=FALSE, strDist, bGraph=FALSE, nNum=10){
   
 
   fast.estimate = FALSE
@@ -158,16 +256,20 @@ KhmaladzeTrans = function(X, strDist, bGraph=FALSE, nNum=10){
   
   StandX = sort(StandX)
   
-  d_alpha= c(2.80705, 2.49771, 2.24241, 1.959, 1.64)
+  NormalMat = Tables$Integration.Table.Normal
+  LogisMat = Tables$Integration.Table.Logistic1
+  ReMat = Tables$Integration.Table.Logistic2
+  CauchyMat = Tables$Integration.Table.Cauchy
   
-  #data(Integration_Tables)
   
-  NormalMat = Integration_Tables$normal.table
-  LogisMat = Integration_Tables$logistic.table1
-  ReMat = Integration_Tables$logistic.table2
-  CauchyMat = Integration_Tables$cauchy.table
+
+  if(Modified==FALSE){
+    bModified=0
+  }else{
+    bModified=1
+  }
   
-  lst = KmtMain(X, NormalMat, LogisMat, ReMat, CauchyMat, strDist, bGraph, nNum)
+  lst = KmtMain(StandX, bModified, NormalMat, LogisMat, ReMat, CauchyMat, strDist, bGraph, nNum)
   
   lineVec = lst[[3]]
   gVec = lst[[4]]
@@ -221,9 +323,15 @@ KhmaladzeTrans = function(X, strDist, bGraph=FALSE, nNum=10){
     labs(title = "Graph of the Objective Function", x="x", y="y")
   
   
-  lResult = list(opt.x = lst[[1]], test.stat=lst[[2]], 
-                 graph.data = DM, graph = gObj, intervals=Intervals, mu = muhat, sigma = sigmahat, 
-                 crit.value = c("1%: 2.80705", "2.5%: 2.49771", "5%: 2.24241", "10%: 1.959") )
+  if(Modified==FALSE){
+    lResult = list(opt.x = lst[[1]], test.stat=lst[[2]], 
+                   graph.data = DM, graph = gObj, intervals=Intervals, mu = muhat, sigma = sigmahat)
+  }else{
+    lResult = list(opt.x = c(lst[[5]], lst[[7]]), test.stat=c(lst[[6]], lst[[8]], (lst[[6]]-lst[[8]])), 
+                   graph.data = DM, graph = gObj, intervals=Intervals, mu = muhat, sigma = sigmahat)
+  }
+  
+
   return(lResult)
 }
 
